require("localenv");
const { exec } = require("node:child_process");
const path = require("path");
const fs = require("fs");
const { Config } = require("../db/db");
const fastFolderSizeSync = require("fast-folder-size/sync");

const DATA_DIR = path.join(__dirname, "..", "..", "data", "backup");
const UPLOADS_DIR = process.env.static_uploads;
const backupSchedulers = [];

module.exports = {
    /**
     * @throws {Error} when fs problem
     * @returns Promise
     */
    async create() {
        let currentBackupPath = path.join(DATA_DIR, new Date().toISOString().replaceAll(":", "#"));
        console.log(currentBackupPath);

        let p = new Promise(function (resolve, reject) {
            let command = process.platform !== "win32" ? 'mongodump --uri="' + process.env.MongoDB_URI + '" --archive=' + currentBackupPath + "/DB.gz --gzip" : __dirname + '\\bin\\mongodump --uri="' + process.env.MongoDB_URI + '" --archive=' + currentBackupPath + "\\DB.gz --gzip";

            let subProcess = exec(command, { timeout: 1000 * 60, cwd: __dirname }, (error, stdout, stderr) => {
                if (error) {
                    console.error(`exec error: ${error}`);
                    error.exitCode = subProcess.exitCode;
                    reject(error);
                    return;
                }
                console.log(`stdout: ${stdout}`);
                console.error(`stderr: ${stderr}`);
                resolve({ exitCode: subProcess.exitCode, stdout, stderr });
            });
        });
        copyFolderSync(UPLOADS_DIR, path.join(currentBackupPath, "uploads"));
        this.removeOlders((await Config.get()).backupsCount);
        return p;
    },
    /**
     * @throws {Error} when fs problem
     * @returns {{date: Date, filename: String, size: Number}[]} array of {date: Date, filename: String, size: Number}
     */
    getAll() {
        dataDir();
        return fs.readdirSync(DATA_DIR).map((f) => {
            const size = fastFolderSizeSync(path.join(DATA_DIR, f));
            return { date: f.replaceAll("#", ":"), filename: f, size };
        });
    },
    /**
     *
     * @param {string} dirName of the backup
     */
    delete(dirName) {
        dirName = dirName.replaceAll(":", "#");
        if (dirName.includes("/") || dirName.includes("\\")) throw Error("Invalid backup filename");
        fs.rmSync(path.join(DATA_DIR, dirName), { recursive: true });
    },
    async schedule() {
        backupSchedulers.forEach((s) => clearInterval(s));
        let now = new Date();
        let night = new Date(
            now.getFullYear(),
            now.getMonth(),
            now.getDate() + 1,
            0, //now.getHours(),
            0, //now.getMinutes() + 1,
            0 //now.getSeconds() + 3 // ...at 00:00:00 hours
        );
        let msToMidnight = night.getTime() - now.getTime();

        let durationMs = +(await Config.get()).backupSchedule * 1000 * 60 * 60 * 24;
        if (!durationMs) return;
        backupSchedulers[0] = setTimeout(() => {
            this.create();
            backupSchedulers[1] = setInterval(() => {
                this.create();
            }, durationMs);
        }, msToMidnight);
    },
    /**
     * @param {Number} count of backups to keep saved, olders will be deleted
     */
    removeOlders(count) {
        count < 1 ? (count = 1) : null;
        let backups = this.getAll().sort((a, b) => Date(a.date) < Date(b.date));
        backups.length = backups.length - count > 0 ? backups.length - count : 0;
        backups.forEach((v) => {
            if (!v) return;
            this.delete(v.filename);
        });
    },
    /**
     * @throws {Error} when fs problem
     * @param {String} backupName name of backup dir
     * @returns Promise
     */
    restore(backupName) {
        backupName = backupName.replaceAll(":", "#");
        let currentBackupPath = path.join(DATA_DIR, backupName);
        console.log("Restore: " + currentBackupPath);

        let p = new Promise(function (resolve, reject) {
            let command = process.platform !== "win32" ? 'mongorestore --uri="' + process.env.MongoDB_URI + '" --archive=' + currentBackupPath + "/DB.gz --gzip --drop" : __dirname + '\\bin\\mongorestore --uri="' + process.env.MongoDB_URI + '" --archive=' + currentBackupPath + "\\DB.gz --gzip --drop";

            let subProcess = exec(command, { timeout: 1000 * 60, cwd: __dirname }, (error, stdout, stderr) => {
                if (error) {
                    console.error(`exec error: ${error}`);
                    error.exitCode = subProcess.exitCode;
                    reject(error);
                    return;
                }
                console.log(`stdout: ${stdout}`);
                console.error(`stderr: ${stderr}`);
                resolve({ exitCode: subProcess.exitCode, stdout, stderr });
            });
        });
        fs.rmSync(UPLOADS_DIR, { recursive: true });
        copyFolderSync(path.join(currentBackupPath, "uploads"), UPLOADS_DIR);
        return p;
    },
};

function copyFolderSync(from, to) {
    fs.mkdirSync(to, { recursive: true });
    fs.readdirSync(from).forEach((element) => {
        if (fs.lstatSync(path.join(from, element)).isFile()) {
            fs.copyFileSync(path.join(from, element), path.join(to, element));
        } else {
            copyFolderSync(path.join(from, element), path.join(to, element));
        }
    });
}

function dataDir() {
    if (!fs.existsSync(DATA_DIR)) fs.mkdirSync(DATA_DIR);
}
