const { Model } = require("mongoose");

module.exports.MenuCache = class MenuCache extends Map {
    /**
     * @type {Model}
     */
    #menuModel;
    #articleModel;
    #localeIndex;
    constructor(menuModel, articleModel, localeIndex) {
        super();
        this.#menuModel = menuModel;
        this.#articleModel = articleModel;
        this.#localeIndex = localeIndex;
    }
    /**
     * @override
     * @param {boolean} throwing wheter to throw Error when some menu contains 404 item
     */
    async clear(throwing = false) {
        let temp = new Map();
        let menus = await this.#menuModel.find({});
        menus = JSON.parse(JSON.stringify(menus));
        for await (const article of this.#articleModel.find({})) {
            menus.forEach((menu) => {
                // modif. menu: add aditional param. (slug, title)
                this.#onEveryItem(menu.items, (item) => {
                    if (article._id.equals(item.articleId)) {
                        item.slug = article.slug;
                        if (!article.translations[this.#localeIndex]) item.title = article.translations[0].title;
                        else item.title = article.translations[this.#localeIndex].title;
                    } else;
                });
            });
        }
        menus.forEach((menu) => temp.set(menu.menuName, menu));

        super.clear();
        for (let [key, value] of temp.entries()) {
            // fill cache with temp cache
            this.set(key, value);
        }
        this.#check(throwing);
    }
    /**
     * @callback exec
     * @param {Object} item
     */
    /**
     * @callback execOnItems
     * @param {Array<Object>} item
     */
    /**
     * @param {Array<Object>} items menuItemSchemas
     * @param {Number} matchingId
     * @param {exec} exec
     * @param {execOnItems} exec executes after iterating through all children
     * @returns
     */
    #onEveryItem(items, exec, execOnItems = () => {}) {
        items.forEach((item) => {
            exec(item);
            this.#onEveryItem(item.children, exec);
        });
        execOnItems(items);
    }
    /**
     * checks the menu so that there are no empty menu items and adds placeholders to these empty items
     * @returns {undefined} if everything is ok
     * @param {boolean} throwing whether to throw Error, otherwise, just fill in the placeholders
     * @throws {Error} if some menu contains empty item
     */
    #check(throwing = false) {
        let errors = new Array();
        this.forEach((menu) => {
            this.#onEveryItem(menu.items, (item) => {
                if (item.articleId && (!item.slug || !item.title)) {
                    if (!item.translations) item.translations = [{}];
                    item.translations[0].title = "Nenalezeno";
                    errors.push([item.articleId, menu.menuName]);
                }
            });
        });
        let err;
        if (errors.length !== 0) {
            err = new Error();
            err.message = "Article ID was not found in menu. More in prop details[['ArticleID', 'MenuName'],...]";
            err.name = "MenuCheckError";
            err.details = errors;
        }
        if (err && throwing) {
            throw err;
        } else if (err && !throwing) console.error(err);
    }
};

module.exports.ArticleCache = class ArticleCache extends Array {
    /**
     * @type {Model}
     */
    #articleModel;
    #localeIndex;
    constructor(articleModel, localeIndex) {
        super();
        this.#articleModel = articleModel;
        this.#localeIndex = localeIndex;
    }
    /**
     * @override
     */
    async clear() {
        let tmp = await this.#articleModel.find({}, { "translations.content": 0, __v: 0, createdAt: 0, updatedAt: 0, allowUserEditor: 0, heading: 0, coverImgPath: 0 }).lean();
        for (const article of tmp) {
            if (article.translations[this.#localeIndex]) Object.assign(article, article.translations[this.#localeIndex] || {});
            else Object.assign(article, article.translations[0] || {});
            article.translations = undefined;
            delete article.translations;
            // console.log(article);
        }
        // console.log(tmp);
        super.splice(0, this.length);
        this.push(...tmp);
    }
};
