require("localenv");
const mongoose = require("mongoose");
const { MenuCache, ArticleCache } = require("./cache");
const schemas = require("./schemas");
const mongoStore = require("connect-mongo");
const { BSON } = require("bson");

const dbConfig = {};

const User = {
    name: "User",
    model: mongoose.model("User", schemas.userSchema),
    fixtures: require("./fixtures").users,
    /**
     * @param {string} email
     * @returns
     */
    get: function (email) {
        return this.model.findOne({ email }).exec();
    },
    /**
     * @returns {Array}
     */
    getAll: async function () {
        const users = await this.model.find({});
        return users.map((user) => ({ email: user.email, role: user.role, size: BSON.calculateObjectSize(user) }));
    },
    /**
     * @param {*} email
     * @returns
     */
    delete: function (email) {
        return this.model.findOneAndDelete({ email });
    },
    /**
     * @param {*} user
     * @returns
     */
    save: function (user) {
        delete user._id;
        return this.model.findOneAndUpdate({ email: user.email }, user, { upsert: true, new: true });
    },
};

// Article
const Article = {
    name: "Article",
    model: mongoose.model("Article", schemas.articleSchema),
    fixtures: require("./fixtures").articles,
    /**
     * @type {ArticleCache}
     */
    cache: [], // declared below
    /**
     * @param {string} articleId
     * @param {number} locale index
     * @returns {Promise<(Object|null)>} null if not found
     */
    get: async function (articleId, locale) {
        const article = await this.model.findById(articleId).lean();
        if (locale !== undefined && article.translations) Object.assign(article, article.translations[locale] || {});
        return article;
    },
    /**
     * @param {string} slug
     * @returns {Promise<(Object|null)>} null if not found
     */
    getBySlug: async function (slug, locale) {
        if (!slug.startsWith("/")) slug = "/" + slug;
        const article = await this.model
            .find({ slug })
            .lean()
            .exec()
            .then((p) => p[0]);
        if (!article.translations) console.log(article);
        if (locale !== undefined) Object.assign(article, article.translations[locale] || {});
        return article;
    },
    //todo
    /**
     * @param {string} slug
     * @returns {Promise<(Object|null)>} null if not found
     */
    deleteBySlug: async function (slug) {
        if (!slug.startsWith("/")) slug = "/" + slug;
        let r = await this.model.findOne({ slug });
        if (!r) return r;
        if (Menu.get(r.slug.replace("/", "").trim())) {
            let err = Error();
            err.name = "CannotDeleteTbMainArticle"; // 406 - Not Acceptable
            err.message = "Toto je učebnice a proto jde smazat pouze přes správce učebnic.";
            throw err;
        } else await this.model.findOneAndDelete({ slug });
        if (cacheToClear) await clearCache({ throwing: true });
        return { status: "done" };
    },
    /**
     * @returns {Array<Object>}
     */
    getAll: function (locale = 0) {
        return this.cache[locale];
    },
    /**
     * @param {String} articleId
     * @returns {Promise<(Object|null)>} null if not found
     */
    delete: async function (articleId, cacheToClear = true) {
        let r = await this.model.findById(articleId);
        if (!r) return r;
        if (Menu.get(r.slug.replace("/", "").trim())) {
            let err = Error();
            err.name = "CannotDeleteTbMainArticle"; // 406 - Not Acceptable
            err.message = "Toto je učebnice a proto jde smazat pouze přes správce učebnic.";
            throw err;
        } else await this.model.findByIdAndDelete(articleId);
        if (cacheToClear) await clearCache({ throwing: true });
        return { status: "done" };
    },
    /**
     * @param {Object} article
     * @returns {Promise<(Object|null)>} null if not found, but it will create it (upsert: true) <= good for testing
     */
    save: async function (article, throwing = true, cacheClear = true) {
        if (article._id) {
            const old = await this.get(article._id);
            const r = await this.model.findByIdAndUpdate(article._id, article, { upsert: global.upsert, new: true });
            // for renaming menu when tb slug changed
            let menu = !old ? null : Menu.get(old.slug.replace("/", "").trim());
            if (menu && old.slug !== r.slug) {
                menu.menuName = r.slug.replace("/", "").trim();
                await Menu.model.findByIdAndUpdate(menu._id, menu);
            }
            if (cacheClear) await clearCache({ menu: false, throwing });
            return r;
        } else {
            const r_1 = await this.model.create(article);
            if (cacheClear) await clearCache({ menu: false, throwing });
            return r_1;
        }
    },
};

// Menu
const Menu = {
    name: "Menu",
    model: mongoose.model("Menu", schemas.menuSchema),
    fixtures: require("./fixtures").menus,
    /**
     * @type {MenuCache}
     */
    cache: [], // declared below
    /**
     * @param {string} menuName
     * @param {number} locale index
     * @returns {(Object|undefined)} undefined if not found
     */
    get: function (menuName, locale) {
        if (this.cache[locale]) return this.cache[locale].get(menuName);
        else return this.cache[0].get(menuName);
    },
    /**
     * @returns {Array<Object>}
     */
    getAll: function (locale = 0) {
        return Array.from(this.cache[locale].values());
    },
    /**
     * @param {string} menuName
     * @throws {Error} if some menu contains empty item (404 placeholder)
     * @returns {Promise<(Object|null)>} null if not found
     */
    delete: async function (menuName) {
        const r = await this.model.findOneAndDelete({ menuName: menuName });
        await clearCache({ article: false, throwing: true });
        return r;
    },
    /**
     * @param {Object} menu
     * @throws {Error} if some menu contains empty item, or the MenuName is not unique
     * @returns {Promise<(Object|null)>} null if not found
     */
    save: async function (menu, throwing = true, cacheClear = true) {
        if (menu._id) {
            delete menu._id;
            const r = await this.model.findOneAndUpdate({ menuName: menu.menuName }, menu, { upsert: global.upsert, new: true });
            if (cacheClear) await clearCache({ article: false, throwing });
            return r;
        } else {
            if ((await this.model.find({ menuName: menu.menuName }).exec()).length === 0) {
                const r = await this.model.create(menu);
                if (cacheClear) await clearCache({ article: false, throwing });
                return r;
            } else {
                let e = new Error("Menu with 'menuName' " + menu.menuName + " already exist");
                e.name = "MenuAlreadyExists";
                throw e;
            }
        }
    },
};

// Activity
const Activity = {
    name: "Activity",
    model: mongoose.model("Activity", schemas.activitySchema),
    fixtures: require("./fixtures").activities,
    /**
     * @param {string} activitySlug
     * @returns {Promise<(Object|null)>} null if not found
     */
    get: function (slug) {
        return this.model.findOne({ slug }).exec();
    },
    /**
     * @param {string} activityId
     * @returns {Promise<(Object|null)>} null if not found
     */
    getById: function (activityId) {
        return this.model.findById(activityId).exec();
    },
    /**
     * @returns {Array<Object>}
     */
    getAll: function () {
        return this.model.find({}, { expiration: 0, questions: 0, cards: 0 });
    },
    /**
     * @param {String} activityID
     * @returns {Promise<(Object|null)>} null if not found
     */
    delete: async function (activityID) {
        //TODO check jestli není aktivita někde referovaná v articlech
        await this.model.findByIdAndDelete(activityID);
        return { status: "done" };
    },
    /**
     * @param {Object} activity
     * @returns {Promise<(Object|null)>} null if not found, but if global upsert: true it will create it <= good for testing
     */
    save: function (activity) {
        if (activity._id) return this.model.findByIdAndUpdate(activity._id, activity, { upsert: global.upsert, new: true });
        else return this.model.create(activity);
    },
};

// G-Config
const Config = {
    name: "Config",
    model: mongoose.model("Config", schemas.globalConfigSchema),
    /**
     * @returns {Promise<Object>} if not found
     */
    get: async function () {
        let r = await this.model.findOne({}).exec();
        if (!r) {
            await this.save({});
            r = await this.model.findOne({}).exec();
        }
        return r;
    },
    /**
     * @param {Object} config
     * @returns {Promise<Object>}
     */
    save: function (config) {
        return this.model.updateOne({}, config, { upsert: true }).exec();
    },
};

/**
 * Function that sets up DB, conditionally fills it with initial data and exit, otherwise creates first caches
 */
async function init() {
    console.group("DB init started");
    try {
        global.upsert = true;

        // DB connect
        mongoose.set("strictQuery", true);
        let dbConnection = await mongoose.connect(process.env.MongoDB_URI).catch((err) => {
            console.error("DB CONNECTION ERROR...\n", err);
            process.exit(1);
        });
        dbConfig.dbConnection = mongoose.connection;
        console.log("DB connected (state: " + dbConfig.dbConnection.readyState + ")");

        // start params
        const addAdmin = async (email) => {
            await User.save({ email: email, name: email.split("@")[0], role: "admin" });
            console.log("Added admin email: " + process.argv[3]);
        };
        switch (process.argv[2]) {
            case "loadFixtures":
                console.group("DB REWRITE WITH FIXTURES");
                const entities = [User, Article, Menu, Activity];
                await Promise.all(
                    entities.map(async (entity) => {
                        await entity.model.deleteMany({});
                        await Promise.all(entity.fixtures.map((fixture) => entity.save(fixture, false, false)));
                        console.log("Saved", entity.fixtures.length, "fixtures (" + entity.name + ")");
                    })
                );
                if (process.argv[3]) await addAdmin(process.argv[3]);
                process.exit(0);
                break;
            case "admin":
                console.group("DB ADD ADMIN");
                await addAdmin(process.argv[3]);
                process.exit(0);
                break;
            case "delete":
                console.group("DB DELETE");
                await new Promise((resolve) => setTimeout(resolve, 500));
                await dbConnection.connection.db.dropDatabase();
                console.log("Database dropped");
                process.exit(0);
        }

        // creating all caches; locales: cs, en, sk, es, de
        for (let localeIndex = 0; localeIndex < 5; localeIndex++) {
            Menu.cache[localeIndex] = new MenuCache(Menu.model, Article.model, localeIndex);
            Article.cache[localeIndex] = new ArticleCache(Article.model, localeIndex);
        }
        await clearCache({ throwing: false, log: true });

        // DB store
        dbConfig.store = mongoStore.create({ mongoUrl: process.env.MongoDB_URI });
        console.log("DB session store created");

        global.upsert = false;
    } catch (err) {
        console.error("DB INIT ERROR...\n" + err);
    }
    console.groupEnd();
}

/**
 * Function for handling all caching requests
 * @param {Object} caches
 * @param {Boolean} caches.article
 * @param {Boolean} caches.menu
 * @param {Boolean} caches.throwing
 * @param {Boolean} caches.log
 * @returns {Promise} if caching was successful
 */
async function clearCache(args = {}) {
    const { article = true, menu = true, throwing = false, log = false } = args;

    let menuCacheError = false;
    try {
        if (article)
            for (const c of Article.cache) {
                await c.clear();
            }
        if (menu)
            for (const c of Menu.cache) {
                await c.clear(throwing);
            }
    } catch (err) {
        menuCacheError = true;
        if (log) console.error("CACHING ERROR...\n", err);
        throw err;
    } finally {
        global.debug
            ? console.table([
                  { cache: "articles", cleared: article, successful: article ? true : "-" },
                  { cache: "menus", cleared: menu, successful: menu ? !menuCacheError : "-" },
              ])
            : console.log([
                  { cache: "articles", cleared: article, successful: article ? true : "-" },
                  { cache: "menus", cleared: menu, successful: menu ? !menuCacheError : "-" },
              ]);
    }
}

module.exports = { dbConfig, User, Article, Menu, Activity, Config, init, clearCache };

async function migrate() {
    let i = 0;
    console.log("migrate");
    for await (const article of Article.model.find({})) {
        i++;
        // if (i > 5) process.exit();
        let k = article.keyWords;
        if (typeof k === "string") k = k.split(", ");
        if (typeof k === "object") k = k[0].split(", ");
        console.log(k);
        article.translations = [];
        article.translations[0] = {
            heading: article.heading,
            title: article.title,
            content: article.content,
            summary: article.summary,
            keyWords: k,
        };
        await article.save();
        await Article.model.findByIdAndUpdate(article._id, { $unset: { heading: true, title: true, content: true, summary: true, keyWords: true } });
    }
    console.log("migrated");
}
