const mongoose = require("mongoose");
const { Schema, ObjectId } = mongoose;

/* -----------------------USER-------------------------------- */
const userSchema = new Schema(
    {
        // _id
        // createdAt
        // updatedAt
        email: { type: String, required: true, unique: true },
        name: { type: String, required: true },
        role: {
            type: String,
            default: "user",
            enum: {
                values: ["dev", "admin", "user"],
                message: '{VALUE} is not supported. Has to be "dev" or "admin" or "user"',
            },
        },
        globalNotes: {
            type: Map,
            of: String,
            default: new Map(),
        },
        dta: [
            {
                articleId: { type: ObjectId, required: true },
                data: {
                    anchorBlocks: [
                        {
                            value: { type: String, required: true },
                            color: { type: String, required: true },
                            data: { type: Object, required: true },
                            anchors: [
                                {
                                    startOffset: { type: Number, required: true },
                                    endOffset: { type: Number, required: true },
                                    xPath: { type: String, required: true },
                                    value: { type: String, required: true },
                                },
                            ],
                        },
                    ],
                },
            },
        ],
        activitiesHistory: [
            {
                activityId: { type: ObjectId, required: true },
                completedAt: { type: String },
                answers: [],
            },
        ],
    },
    { timestamps: true, minimize: false }
);
module.exports.userSchema = userSchema;

/* -----------------------ARTICLE-------------------------------- */
module.exports.articleSchema = new Schema(
    {
        // _id
        // createdAt
        // updatedAt
        slug: { type: String, required: true, unique: true },
        coverImgPath: { type: String, required: true },
        // published: { type: Boolean, required: true },
        allowUserEditor: { type: Boolean, required: true },
        activities: { type: [ObjectId], default: [] },
        translations: {
            type: Array,
            of: {
                heading: { type: String }, // <h1>
                title: { type: String, maxLength: 100 }, // text in menu
                content: { type: String },
                summary: { type: String },
                keyWords: { type: [String], default: [] },
                _id: false,
            },
            default: [],
            required: true,
        },
    },
    { timestamps: true }
);
module.exports.articleSchema.index({ "translations.heading": "text", "translations.title": "text", "translations.content": "text", "translations.keyWords": "text" });

/* -----------------------MENU-------------------------------- */
const menuItemSchema = new Schema({});
menuItemSchema.add({
    // _id
    articleId: {
        type: ObjectId,
        required: function () {
            return !isNode.call(this);
        },
    }, // id of the article to which the link should lead
    children: [
        {
            type: menuItemSchema,
            required: function () {
                return !this.articleId;
            },
            validate: {
                validator: function (v) {
                    return !this.articleId && v.length > 0;
                },
                message: (props) => props.value + " contains articleId, but also constains children and it is not valid.",
            },
        },
    ],
    title: {
        type: String,
        required: function () {
            return isNode.call(this);
        },
    },
    type: {
        type: String,
        required: true,
        enum: {
            values: ["tb", "a", "n"], // tb = textbook, a = article, n = node = root of dropdown menu
            message: '{VALUE} is not supported. Has to be "tb" or "a" or "n"',
        },
    },
});
function isNode() {
    return this.children.length !== 0;
    // if it does not have children, see the validate function
}
module.exports.menuSchema = new Schema(
    {
        // _id
        // createdAt
        // updatedAt
        menuName: { type: String, required: true, unique: true },
        items: [menuItemSchema],
    },
    { timestamps: true }
);

/* -------------------ACTIVITIES------------------------------------ */
const questionSchema = new Schema(
    {
        // createdAt
        // updatedAt
        text: { type: String, required: true },
        img: {
            type: String,
            required: function () {
                return this.type === "img_description";
            },
        }, // path to img
        shuffleAnswers: { type: Boolean, required: true },
    },
    { discriminatorKey: "type", _id: false, timestamps: true }
);
const questionSelectSchema = new Schema({
    answers: {
        type: [
            {
                text: { type: String, required: true },
                correct: { type: Boolean, required: true },
            },
        ],
        required: true,
    },
});
const questionMultipleSelectSchema = new Schema({
    answers: {
        type: [
            {
                text: { type: String, required: true },
                correct: { type: Boolean, required: true },
            },
        ],
        required: true,
    },
});
const questionImgDescriptionSchema = new Schema({
    answers: {
        type: [
            {
                text: { type: String, required: true },
                number: { type: Number, required: true },
            },
        ],
        required: true,
    },
});
const flashcardSchema = new Schema(
    {
        // createdAt
        // updatedAt
        sides: [
            {
                text: { type: String, required: true },
                img: { type: String, default: "" }, // path to img
            },
            {
                text: { type: String, required: true },
                img: { type: String, default: "" }, // path to img
            },
        ],
    },
    { _id: false, timestamps: true }
);
const activitySchema = new Schema(
    {
        //_id
        // createdAt
        // updatedAt
        type: {
            type: String,
            required: true,
            enum: {
                values: ["test", "quiz", "flashcards"],
                message: '{VALUE} is not supported. Has to be "test" or "quiz" or "flashcards"',
            },
        },
        title: { type: String, required: true },
        description: { type: String, default: "" },
        expiration: {
            type: String,
            required: false,
        },
        slug: { type: String, required: true, unique: true },
        questions: {
            type: [questionSchema],
            required: function () {
                return this.type === "test" || this.type === "quiz";
            },
        },
        cards: {
            type: Array,
            of: flashcardSchema,
            required: function () {
                return this.type === "flashcards";
            },
        },
    },
    { timestamps: true }
);
activitySchema.path("questions").discriminator("select", questionSelectSchema);
activitySchema.path("questions").discriminator("multiple_select", questionMultipleSelectSchema);
activitySchema.path("questions").discriminator("img_description", questionImgDescriptionSchema);
module.exports.activitySchema = activitySchema;

/* -----------------------GLOBAL CONFIG-------------------------------- */
module.exports.globalConfigSchema = new Schema(
    {
        // _id
        // createdAt
        // updatedAt
        author: { type: String, default: "Petr Chalupa & Martin Voplakal & Filip Beneš" },
        webName: { type: String, default: "Digital Education Hub" },
        contact: { email: { type: String, default: "ekdyson.help@gmail.com" }, name: { type: String, default: "DEV" } },
        logoFilename: { type: String, default: "/defaults/defaultLogo.webp" },
        faviconFilename: { type: String, default: "/defaults/defaultFavicon.webp" },
        backupSchedule: { type: Number, default: 1 },
        AiFreeTokens: { type: Number, required: true, default: process.env.AI_TOKENS_BUDGET },
        backupsCount: { type: Number, default: 10 },
        footer: {
            leftCol: {
                type: String,
                default:
                    '<p>Vývojáři:</p>\n<ul>\n<li><a href="mailto:petr.chalupa@student.gyarab.cz">PETR CHALUPA</a></li>\n<li><a href="mailto:filip.benes@student.gyarab.cz">FILIP BENE&Scaron;</a></li>\n<li><a href="mailto:martin.voplakal@student.gyarab.cz">MARTIN VOPLAKAL</a></li>\n</ul>',
            },
            rightCol: {
                type: String,
                default: "<address>\n<p>Adresa:</p>\n<p>Arabská 682/14, Praha 6 - Vokovice</p>\n<p>235 351 707</p>\n</address>",
            },
        },
        colorSchemas: {
            selected: { type: Number, default: 0 },
            schemas: {
                type: Array,
                of: Object,
                default: [
                    {
                        colors: [
                            { name: "bg-primary", light: "#ffffff", dark: "#373331" },
                            { name: "bg-secondary", light: "#93deff", dark: "#008bc9" },
                            { name: "bg-tertiary", light: "#606470", dark: "#303238" },
                            { name: "bg-quaternary", light: "#f7f7f7", dark: "#413f3e" },
                            { name: "bg-quiz", light: "#fed766", dark: "#fecb34" },
                            { name: "bg-test", light: "#fe4a49", dark: "#d32a2a" },
                            { name: "bg-flashcard", light: "#009fb8", dark: "#009fb8" },
                            { name: "color-details", light: "#e7e7e7", dark: "#d1d1d1" },
                        ],
                        immutable: true,
                    },
                    {
                        colors: [
                            { name: "bg-primary", light: "#ffffff", dark: "#373331" },
                            { name: "bg-secondary", light: "#ffb8d5", dark: "#db6b97" },
                            { name: "bg-tertiary", light: "#6bcb77", dark: "#7eaa65" },
                            { name: "bg-quaternary", light: "#f2f2f2", dark: "#423f3e" },
                            { name: "bg-quiz", light: "#e63e6d", dark: "#781d42" },
                            { name: "bg-test", light: "#f64b3c", dark: "#dc2f2f" },
                            { name: "bg-flashcard", light: "#1da2f5", dark: "#1da2f5" },
                            { name: "color-details", light: "#e7e7e7", dark: "#f7f7f7" },
                        ],
                        immutable: true,
                    },
                    {
                        colors: [
                            { name: "bg-primary", light: "#ffffff", dark: "#000000" },
                            { name: "bg-secondary", light: "#000000", dark: "#ffffff" },
                            { name: "bg-tertiary", light: "#ffffff", dark: "#000000" },
                            { name: "bg-quaternary", light: "#ffffff", dark: "#000000" },
                            { name: "bg-quiz", light: "#ffffff", dark: "#000000" },
                            { name: "bg-test", light: "#ffffff", dark: "#000000" },
                            { name: "bg-flashcard", light: "#ffffff", dark: "#000000" },
                            { name: "color-details", light: "#000000", dark: "#ffffff" },
                        ],
                        immutable: false,
                    },
                ],
            },
        },
    },
    { timestamps: true }
);
