require("localenv");
const express = require("express");
const router = express.Router();
const { Config } = require("../db/db");
const { verifyRequestRole } = require("./auth");
const summary = require("../APIs/summary");
const deepL = require("../APIs/deepL");

router.post("/summary", async (req, res, next) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    let text = req.body.text;
    try {
        if (!text) throw Error("body.text not provided");
        try {
            await AITokens(-1);
        } catch (e) {
            return res.status(500).send({ err: e.message });
        }
        res.send({ summary: await summary(text) });
    } catch (e) {
        res.status(500).send({ err: e.message });
    }
});

/**
 * Request
 * body:
 * {
 *  source: "Text"
 *  lang: "en"
 * }
 * nebo
 * {
 *  source: {heading, content, keyWords}
 *  lang: "en"
 * }
 * @returns
 * {
 *  translation: "Text"
 *  lang: "en"
 * }
 * nebo
 * {
 *  translation: {heading, content, keyWords}
 *  lang: "en"
 * }
 */
router.post("/translate", async (req, res, next) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    /**
     * @type {String | Object}
     */
    let source = req.body.source;
    /**
     * supported languages: https://www.deepl.com/docs-api/general/get-languages
     * e.g. en, en-us, en-gb, ...
     */
    let lang = req.body.lang;
    try {
        if (!source) throw Error("body.source not provided");
        if (!lang) throw Error("body.lang not provided");
        try {
            await AITokens(-1);
        } catch (e) {
            return res.status(500).send({ err: e.message });
        }

        if (typeof source === "object") {
            let translation = {};
            for (const prop in source) {
                if (source[prop].length != 0) translation[prop] = deepL.translate(source[prop], lang);
            }
            for (const prop in source) {
                translation[prop] = await translation[prop];
            }
            res.send({ translation });
        } else res.send({ translation: await deepL.translate(source, lang) });
    } catch (e) {
        res.status(500).send({ err: e.message });
    }
});

async function AITokens(count) {
    let config = await Config.get();
    if (config.AiFreeTokens <= 0) throw Error("Your usage of summary feature exceeded our monthly limit. Please contact us and we can increase it.");
    config.AiFreeTokens += count;
    await config.save();
}

async function resetTokens() {
    let config = await Config.get();
    config.AiFreeTokens = Number(process.env.AI_TOKENS_BUDGET);
    await config.save();
}

async function init() {
    console.group("AI init started");
    try {
        // monthly reset AiTokens usage
        resetTokens();
        let schedule = require("node-schedule");
        let rule = new schedule.RecurrenceRule();
        rule.month = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11];
        rule.date = 1;
        rule.hour = 0;
        rule.minute = 0;
        schedule.scheduleJob(rule, resetTokens);
        console.log("Tokens reset scheduled");
    } catch (err) {
        console.error("AI INIT ERROR...\n" + err);
    }
    console.groupEnd();
}

module.exports = { router, init };
