require("localenv");
const express = require("express");
const router = express.Router();
const jwt = require("jsonwebtoken");
const { google } = require("googleapis");
const db = require("../db/db");

const GoogleAPIs = {
    gmail: null,
};

async function checkAuthToken(req, res, next) {
    const { authToken } = req.query;
    if (authToken) {
        try {
            const data = jwt.verify(authToken, process.env.JWT_SECRET, {});
            if (data.iss != "Ekdyson-Auth-Server") throw Error("Auth token was not provided by a trusted issuer (" + data.iss + ")");

            const user = await db.User.save(data);
            req.session.user = { email: user.email, name: user.name, role: user.role };

            let continueURL = req.originalUrl.split("?").shift();
            if (process.env.NODE_ENV === "development") {
                continueURL = `${req.protocol}://${req.headers.host}${req.originalUrl.split("?").shift()}`;
                continueURL = continueURL.replace(new RegExp(":" + process.env.PORT), ":8080");
            }
            return res.redirect(continueURL);
        } catch (error) {
            return res.status(500).send({ err: error.message });
        }
    }
    next();
}

router.get("/login", async (req, res) => {
    let serviceURL = `${req.protocol}://${req.headers.host}`;
    if (process.env.NODE_ENV === "development") serviceURL = serviceURL.replace(/:\d{4}/, ":" + process.env.PORT);
    const successRedirectURL = serviceURL + (req.query.successRedirectURL ?? "/");
    const failureRedirectURL = serviceURL + (req.query.failureRedirectURL ?? "/err");
    return res.redirect(`${process.env.AUTH_SERVER_URL}?successRedirectURL=${successRedirectURL}&failureRedirectURL=${failureRedirectURL}`);
});

router.get("/session", async (req, res) => {
    if (req.session.user) {
        const user = await db.User.get(req.session.user.email);
        res.send(user);
    } else res.status(401).send({ err: "Unauthorized (session)" });
});

router.post("/logout", async (req, res) => {
    req.session.destroy(() => {
        res.send({});
    });
});

/**
 * Function for verification of admin access
 * @param {Object} req
 * @param {Object} res
 * @param {String|Array} roles
 * @returns {Boolean} true if the user is authorized
 */
function verifyRequestRole(req, res, roles) {
    if (typeof roles === "string") roles = [roles];
    const authorized = roles.find((role) => req.session.user.role === role);
    if (authorized) return true;
    else {
        res.status(401).send({ err: "Unauthorized" });
        return false;
    }
}

/**
 * Function initializating GOAuth2Client and required APIs
 */
async function init() {
    console.group("AUTH init started");
    try {
        console.log(`USING SERVICE EMAIL <${process.env.SERVICE_EMAIL}>`);

        // Gmail API
        const googleClient = new google.auth.OAuth2(process.env.GOAuth_CLIENT_ID, process.env.GOAuth_CLIENT_SECRET);
        googleClient.setCredentials({ refresh_token: process.env.GOAuth_CLIENT_REFRESH_TOKEN });
        GoogleAPIs.gmail = google.gmail({ version: "v1", auth: googleClient });
        console.log("GMAIL API set");
    } catch (err) {
        console.error("AUTH INIT ERROR...\n" + err);
    }
    console.groupEnd();
}

module.exports = { router, init, verifyRequestRole, checkAuthToken, GoogleAPIs };
