require("localenv");
const express = require("express");
const router = express.Router();
const { verifyRequestRole } = require("./auth");
const db = require("../db/db");

// Article
router.get("/article/all", async (req, res, next) => {
    let r = await db.Article.getAll(req.header("locale"));
    if (!r) {
        res.status(404).send({ err: "Articles not found" });
    } else if (!res.headersSent) res.send(r);
});
router.get("/article/:id", async (req, res, next) => {
    try {
        let r = await db.Article.get(req.params.id, req.header("locale"));
        if (!r) {
            return res.status(404).send({ err: "Article not found" });
        } else {
            res.send(r);
        }
    } catch (e) {
        if (e.name === "CastError") return res.status(404).send({ err: e.message });
        else return next(e);
    }
});
router.get("/article/slug/:slug", async (req, res, next) => {
    let r = await db.Article.getBySlug(req.params.slug, req.header("locale")).catch((e) => next(e));
    if (!r) {
        res.status(404).send({ err: "Article not found" });
    } else if (!res.headersSent) res.send(r);
});
router.post("/article", async (req, res, next) => {
    // when creating article, the _id must not be preset
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    try {
        let r = await db.Article.save(req.body);
        if (!r && !res.headersSent) {
            res.status(404).send({ err: "Article not found" });
        } else if (!res.headersSent) res.send(r);
    } catch (e) {
        switch (e.name) {
            case "CastError":
                res.status(404).send({ err: e.message });
                break;
            case "ValidationError":
                res.status(400).send({ err: e.message });
                break;
            default:
                next(e);
        }
    }
});
router.delete("/article/:id", async (req, res, next) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;

    let r = await db.Article.delete(req.params.id).catch((e) => {
        switch (e.name) {
            case "CastError":
                res.status(404).send({ err: e.message });
                break;
            case "MenuCheckError":
                if (e.details.toString().includes(req.params.id)) {
                    let menus = [];
                    e.details.forEach((detail) => {
                        menus.push(detail[1]);
                    });
                    res.status(409).send({
                        //err: "Tento článek byl obsažen v menu: " + menus.join(", ") + ".",
                        err: { menus },
                    });
                } else return { status: "done", err: e };
                break;
            case "CannotDeleteTbMainArticle":
                res.status(406).send({ err: e.message });
                break;
            default:
                next(e);
        }
    });
    if (!r && !res.headersSent) {
        res.status(404).send({ err: "Article not found" });
    } else if (!res.headersSent) res.send(r);
});

//todo
router.delete("/article/slug/:slug", async (req, res, next) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    let r = await db.Article.deleteBySlug(req.params.slug).catch((e) => {
        switch (e.name) {
            case "CastError":
                res.status(404).send({ err: e.message });
                break;
            case "MenuCheckError":
                if (e.details.toString().includes(req.params.id)) {
                    let menus = [];
                    e.details.forEach((detail) => {
                        menus.push(detail[1]);
                    });
                    res.status(409).send({
                        //err: "Tento článek byl obsažen v menu: " + menus.join(", ") + ".",
                        err: { menus },
                    });
                } else return { status: "done", err: e };
                break;
            case "CannotDeleteTbMainArticle":
                res.status(406).send({ err: e.message });
                break;
            default:
                next(e);
        }
    });
    if (!r && !res.headersSent) {
        res.status(404).send({ err: "Article not found" });
    } else if (!res.headersSent) res.send(r);
});

// Menu
router.get("/menu/all", async (req, res) => {
    let r = db.Menu.getAll(req.header("locale"));
    if (!r) {
        res.status(404).send({ err: "Menus not found" });
    } else if (!res.headersSent) res.send(r);
});
router.get("/menu/:menuName", (req, res) => {
    let r = db.Menu.get(req.params.menuName, req.header("locale"));
    if (r) res.send(r);
    else res.status(404).send({});
});
router.post("/menu", async (req, res, next) => {
    // when creating menu, the _id must not be present
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    let r = await db.Menu.save(req.body).catch((e) => {
        switch (e.name) {
            case "CastError":
                res.status(404).send({ err: e.message });
                break;
            case "MenuAlreadyExists":
                res.status(400).send({ err: e.message });
                break;
            case "ValidationError":
                res.status(400).send({ err: e.toString() });
                break;
            case "MenuCheckError":
                let menus = [];
                e.details.forEach((detail) => {
                    menus.push(detail[1]);
                });
                res.status(409).send({
                    //invalid links in menus
                    err: { menus: menus },
                });
                break;
        }
        if (!res.headersSent) next(e);
    });
    if (!r && !res.headersSent) {
        res.status(404).send({ err: "Menu with name: " + req.body.menuName + " not found" });
    } else if (!res.headersSent) res.send(r);
});
/**
 * remove undefined (404) from all menus
 */
router.post("/menu/clean", async (req, res, next) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    let removed = 0;
    function clean(items) {
        let index = items.length - 1;
        while (index >= 0) {
            let item = items[index];
            if (item.articleId && !item.slug) {
                items.splice(index, 1);
                removed++;
            }
            index -= 1;

            if (!item.children) return;
            clean(item.children);
        }
    }
    let promises = [];

    db.Menu.getAll().forEach((menu) => {
        clean(menu.items);
        promises.push(
            db.Menu.save(menu, false, false).catch((e) => {
                if (!res.headersSent) next(e);
                console.error("Menu clean error \n" + e);
            })
        );
    });
    for (const promise of promises) {
        await promise;
    }
    await db.clearCache();
    res.send({ status: "done", count: removed });
});
router.delete("/menu/:menuName", async (req, res, next) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    let r = await db.Menu.delete(req.params.menuName).catch((e) => (e.name === "CastError" ? res.status(404).send({ err: e.message }) : next(e)));
    if (!r && !res.headersSent) {
        res.status(404).send({ err: "Menu not found" });
    } else if (!res.headersSent) res.send(r);
});

//TextBook (todo)
router.delete("/tb/:menuName", async (req, res, next) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    let r = await db.Menu.delete(req.params.menuName).catch((e) => (e.name === "CastError" ? res.status(404).send({ err: e.message }) : next(e)));
    if (res.headersSent) return;
    if (!r) {
        return res.status(404).send({ err: "Menu not found" });
    }
    let a = await Promise.all(r.items.map((item) => db.Article.delete(item.articleId, { clearCache: false }))).catch((e) => next(e));
    await clearCache({ throwing: true });
    if (!res.headersSent) res.send({ deletedMenu: r, deletedArticles: a });
});

/**
 * search
 * returns array of articles
 */
router.get("/search/:query", async (req, res, next) => {
    // todo get languageindex;
    let locale = 0;
    let as = await db.Article.model
        .find({
            $text: {
                $search: req.params.query,
                $caseSensitive: false,
                $diacriticSensitive: false,
            },
        })
        .sort({ score: { $meta: "textScore" } })
        .limit(10)
        .lean()
        .exec();
    for (const a of as) {
        Object.assign(a, a.translations[locale] || {});
        delete a.translations;
    }
    res.send(as);
});

module.exports = { router };
