require("localenv");
const express = require("express");
const router = express.Router();
const fs = require("fs");
const { verifyRequestRole } = require("./auth");
const builder = require("../builder/builder");
const backup = require("../backup/backup");
const { uploadSingleImg } = require("./img");
const { Config } = require("../db/db");
const { sendMail } = require("./user");

router.get("/general", async (req, res) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    const { webName, contact, colorSchemas, footer } = await Config.get();
    res.send({ webName, contact, colorSchemas, footer });
});
router.post("/general", async (req, res, next) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    await builder.modify((data) => {
        data.webName = req.body.newName;
        //
        const oldEmail = data.contact.email;
        const newEmail = req.body.contact.email;
        data.contact = req.body.contact;
        if (oldEmail != newEmail) {
            sendMail({
                from: { name: `GLOBAL SETTINGS | ${data.webName}`, email: process.env.SERVICE_EMAIL },
                to: newEmail,
                replyTo: process.env.SERVICE_EMAIL,
                subject: "Change of email in global settings",
                body: `
                <div>
                    <h2>Your contact email has changed, ${req.body.contact.name}</h2>
                    <p><a href="mailto:${oldEmail}">${oldEmail}</a> → <a href="mailto:${newEmail}">${newEmail}</a></p>
                    <p>This email will now be used to communicate with you, including the contact form.</p>
                </div>
            `,
            }).catch((e) => next(e));
        }
        //
        data.colorSchemas = req.body.colorSchemas;
        //
        data.footer = req.body.footer;
    });
    res.send({});
});

router.get("/logo", async (req, res) => {
    const config = await Config.get();
    let webLogo = fs.readdirSync(process.env.static_uploads).find((file) => "/uploads/" + file === config.logoFilename);
    if (!webLogo) {
        webLogo = "/defaults/defaultLogo.webp";
        if (config.logoFilename !== webLogo) builder.modify((d) => (d.logoFilename = webLogo));
    } else webLogo = "/uploads/" + webLogo;
    res.send({ webLogo });
});
router.post("/logo", async (req, res) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    uploadSingleImg({
        req,
        filename: "logo",
        callback: async (err, filename) => {
            if (err) return res.status(500).send({ err });
            await builder.modify((d) => (d.logoFilename = "/uploads/" + filename));
            res.send({});
        },
    });
});

router.get("/favicon", async (req, res) => {
    const config = await Config.get();
    let webFavicon = fs.readdirSync(process.env.static_uploads).find((file) => "/uploads/" + file === config.faviconFilename);
    if (!webFavicon) {
        webFavicon = "/defaults/defaultFavicon.webp";
        if (config.faviconFilename !== webFavicon) builder.modify((d) => (d.faviconFilename = webFavicon));
    } else webFavicon = "/uploads/" + webFavicon;
    res.send({ webFavicon });
});
router.post("/favicon", async (req, res) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    uploadSingleImg({
        req,
        filename: "favicon",
        callback: async (err, filename) => {
            if (err) return res.status(500).send({ err });
            await builder.modify((d) => (d.faviconFilename = "/uploads/" + filename));
            res.send({});
        },
    });
});

router.get("/backup", async (req, res, next) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    try {
        res.send({ files: backup.getAll() });
    } catch (error) {
        next(error);
    }
});
router.post("/createBackup", async (req, res, next) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    backup
        .create()
        .then((r) => res.send(r))
        .catch((e) => next(e));
});
router.post("/restoreBackup/:name", async (req, res, next) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    if (!req.params.name) return res.status(404).send({ err: "Backup name not specified" });
    backup
        .restore(req.params.name)
        .then((r) => res.send(r))
        .catch((e) => next(e));
});
router.delete("/backup/:name", async (req, res, next) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    try {
        backup.delete(req.params.name);
        res.send({});
    } catch (error) {
        next(error);
    }
});
router.get("/backupSchedule", async (req, res, next) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    try {
        res.send({ days: (await Config.get()).backupSchedule });
    } catch (error) {
        next(error);
    }
});
router.post("/backupSchedule/:days", async (req, res, next) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    if (!req.params.days || req.params.days < 0 || req.params.days > 24) return res.status(400).send({});
    await builder.modify((v) => (v.backupSchedule = req.params.days));
    backup.schedule();
    res.send({});
});
router.get("/backupsCount", async (req, res, next) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    try {
        res.send({ backupsCount: (await Config.get()).backupsCount });
    } catch (error) {
        next(error);
    }
});
router.post("/backupsCount/:count", async (req, res, next) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    if (!req.params.count || req.params.count < 0) return res.status(400).send({});
    await builder.modify((v) => (v.backupsCount = req.params.count));
    // backup.removeOlders(req.params.count);
    res.send({});
});

module.exports = { router };
