require("localenv");
const express = require("express");
const router = express.Router();
const path = require("path");
const fs = require("fs");
const multer = require("multer");
const { verifyRequestRole } = require("./auth");

const tinify = require("tinify");
const { Article, Config } = require("../db/db");
tinify.key = process.env.TinyPNG_API_KEY;

router.get("/all", async (req, res) => {
    const defaults = fs.readdirSync(process.env.static_defaults);
    const uploads = fs.readdirSync(process.env.static_uploads);
    res.send({ defaults, uploads });
});

router.post("/", async (req, res) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    uploadSingleImg({
        req,
        callback: (err, filename) => {
            if (err) return res.status(500).send({ err });
            res.send({ location: "/uploads/" + filename, filename });
        },
    });
});

router.delete("/del/:name", async (req, res) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;

    const imgPath = path.join(process.env.static_uploads, req.params.name);
    if (!fs.existsSync(imgPath)) return res.status(404).send({ err: "Image not found" });

    fs.unlinkSync(imgPath);
    res.send({});
});

router.delete("/unused", async (req, res) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    const { err, deleted } = await deleteAllImages(true);
    if (err) res.status(500).send({ err });
    else res.send({ deleted });
});

/**
 * Function for uploading a single img
 * @param {Object} payload
 * @param {Object} payload.req request
 * @param {String} payload.filename override original filename
 * @param {String} payload.dest destination
 * @param {Boolean} payload.useTinify
 * @param {function} payload.callback callback
 * @returns {String} filename - whole filename with extension
 */
function uploadSingleImg({ req, filename, dest = process.env.static_uploads, useTinify = true, callback = () => {} }) {
    const upload = multer({
        storage: multer.diskStorage({
            destination: dest,
            filename: (req, file, cb) => {
                const ext = file.originalname.split(".").pop();
                let name = filename || file.originalname.split(".").shift();
                //
                const existing = fs.readdirSync(dest).filter((f) => f.split(".").shift().includes(name) && f.split(".").pop() === ext);
                if (existing.length > 0) name += "(" + existing.length + ")";
                //
                filename = name + "." + ext;
                cb(null, filename);
            },
        }),
        limits: { fileSize: 1024 * 1000 * 10 },
    }).single("img");
    upload(req, null, (err) => {
        if (err) {
            if (err instanceof multer.MulterError) err = "Multer err:\n" + err.message;
            console.error("Img saving err:", err);
        } else if (useTinify && ["jpg", "png", "webp"].some((v) => filename.split(".").pop().toLocaleLowerCase().includes(v))) {
            const pathToFile = path.join(dest, filename);
            tinify.fromFile(pathToFile).toFile(pathToFile, (tinifyErr) => {
                if (tinifyErr instanceof tinify.AccountError) {
                    tinifyErr = "Account err (Verify your API key and account limit): " + tinifyErr;
                } else if (tinifyErr instanceof tinify.ClientError) {
                    tinifyErr = "Client err (Check your source image and request options): " + tinifyErr;
                } else if (tinifyErr instanceof tinify.ServerError) {
                    tinifyErr = "Server err (Temporary issue with the Tinify API): " + tinifyErr;
                } else if (tinifyErr instanceof tinify.ConnectionError) {
                    tinifyErr = "Connection err (A network connection error occurred): " + tinifyErr;
                }
                if (tinifyErr) {
                    console.error("Tinify err:", tinifyErr);
                    err = tinifyErr;
                }
            });
        }
        callback(err, filename);
    });
}

/**
 * Function for deleting all uploaded images
 * @param {Boolean} onlyUnused - used images won't be deleted
 * @returns {Object} object with an err or array of deleted images
 */
async function deleteAllImages(onlyUnsused = true) {
    console.group("Deleting", onlyUnsused ? "unused" : "all", "images");
    try {
        let deleted = [];
        let files = fs.readdirSync(process.env.static_uploads);
        for (const file of files) {
            if (onlyUnsused) {
                console.log(file);
                if (!(await isImageUsed(file))) {
                    fs.unlinkSync(path.join(process.env.static_uploads, file));
                    deleted.push(file);
                    console.log("Deleted:", file);
                }
            } else {
                fs.unlinkSync(path.join(process.env.static_uploads, file));
                deleted.push(file);
                console.log("Deleted:", file);
            }
        }
        console.groupEnd();
        return { deleted };
    } catch (err) {
        console.error("IMG DELETION ERROR...\n" + err);
        console.groupEnd();
        return { err };
    }
}

function escapeRegExp(string) {
    return string.replace(/[.*+?^${}()|[\]\\]/g, "\\$&"); // $& means the whole matched string from: https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/Regular_Expressions#escaping
}
/**
 * @returns { Promise<boolean>} if given image is not used anywhere
 */
async function isImageUsed(imagName) {
    if (!imagName) throw Error("imageName not specified");
    let regExp = { $regex: escapeRegExp(imagName), $options: "i" };
    return Boolean(
        (await Article.model.findOne({ $or: [{ coverImgPath: regExp }, { content: regExp }] }).exec()) || (await Config.model.findOne({ $or: [{ faviconFilename: regExp }, { logoFilename: regExp }] }))
    );
}

module.exports = { router, uploadSingleImg, deleteAllImages };
