require("localenv");
const express = require("express");
const router = express.Router();
const { verifyRequestRole, GoogleAPIs } = require("./auth");
const db = require("../db/db");

router.get("/all", async (req, res) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    res.send(await db.User.getAll());
});

router.post("/", async (req, res) => {
    if (!req.body.email || req.session.user?.email !== req.body.email) {
        return res.status(401).send({ err: "Unauthorized" });
    }
    delete req.body.role;
    delete req.body.__v;
    let r = await db.User.save(req.body);
    if (!r) {
        res.status(404).send({ err: "User not found" });
    } else if (!res.headersSent) res.send(r);
});

router.post("/role", async (req, res) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    let r = await db.User.save({ email: req.body.email, role: req.body.role });
    if (!r) {
        res.status(404).send({ err: "User not found" });
    } else if (!res.headersSent) res.send(r);
});

router.delete("/:email", async (req, res) => {
    if (!verifyRequestRole(req, res, ["dev", "admin"])) return;
    let r = db.User.delete(req.params.email);
    try {
        await r;
        if (!r) throw Error("User not found");
        res.send({ userEmail: r.email });
    } catch (err) {
        res.status(500).send({ err });
    }
});

router.post("/sendContactMail", async (req, res, next) => {
    const { loggedIn, name, email, phone, message } = req.body;

    const { webName, contact } = await db.Config.get();
    if (!contact.email) return next(new Error("Contact (reciever) email not found or set"));

    sendMail({
        from: { name: `CONTACT FORM | ${webName}`, email: process.env.SERVICE_EMAIL },
        to: contact.email,
        replyTo: email,
        subject: "Message from a contact form",
        body: `
            <div>
                <p style="font-size: 1rem;">
                    From <b>${name}</b> (${loggedIn ? "Logged in user" : "Not logged in user"})
                </p>
                <p>Email: <a href="mailto:${email}">${email}</a><p>
                <p>Tel.: ${phone ? `<a href="tel:${phone}">${phone}</a>` : "undefined"}</p>
                <pre style="border: 1px solid; border-radius: 0.25rem; padding: 0.5rem;">${message}</pre>
            </div>
        `,
    })
        .then((response) => res.send(response))
        .catch((e) => next(e));
});

/**
 * Function for safe sending email with GMAIL API
 * @param {Object} payload
 * @param {Object} payload.from name and email address
 * @param {String} payload.to email address
 * @param {String} payload.replyTo email address
 * @param {Boolean} payload.subject email subject
 * @param {function} payload.body email body (HMTL/TXT)
 * @returns {Object} id of the sent email
 * @throws {Error}
 */
async function sendMail({ from, to, replyTo, subject, body } = {}) {
    try {
        const utf8Encode = (raw) => `=?utf-8?B?${Buffer.from(raw).toString("base64")}?=`;
        const messageParts = [`From: ${utf8Encode(from.name)} <${from.email}>`, `To: ${to}`, `Reply-To: ${replyTo}`, `Subject: ${utf8Encode(subject)}`, "Content-Type: text/html; charset=utf-8", body];
        const message = messageParts.join("\n");
        const encodedMessage = Buffer.from(message).toString("base64").replace(/\+/g, "-").replace(/\//g, "_").replace(/=+$/, "");
        const { data: { id } = {} } = await GoogleAPIs.gmail.users.messages.send({
            userId: "me",
            requestBody: {
                raw: encodedMessage,
            },
        });

        return { id };
    } catch (err) {
        throw Error("Error sending email: " + err);
    }
}

module.exports = { router, sendMail };
