const express = require("express");
const session = require("express-session");
const cors = require("cors");
const path = require("path");
const fs = require("fs");
const { checkAuthToken } = require("./routes/auth");
const app = express();

// env variables
process.env.data_dir = path.join(__dirname, "..", "data");
process.env.static_public = path.join(__dirname, "vue_build");
process.env.static_defaults = path.join(__dirname, "defaults");
process.env.static_uploads = path.join(process.env.data_dir, "uploads");
process.env.static_files = path.join(__dirname, "files");
// load and inject the variables into process.env
(() => {
    try {
        const envFile = path.join(process.env.data_dir, ".env");
        fs.accessSync(envFile, fs.constants.F_OK);
        const localenv = require("localenv/noload");
        localenv.inject_env(envFile);
    } catch (err) {
        console.error("ERROR: file .env was not found. Add this configuration file and try again.\n", err.message);
        process.exit(1);
    }
})();
process.env.NODE_ENV ??= "development";
global.debug = process.env.NODE_ENV === "development";
console.log("Running in", process.env.NODE_ENV);

async function init() {
    console.group("INIT started");
    try {
        // logging with a timestamp
        if (!global.debug) {
            console.error = (() => {
                let err = console.error;
                return (...args) => err("|" + new Date().toLocaleString("en-GB") + "| ", ...args);
            })();
            console.log = (() => {
                let log = console.log;
                return (...args) => log("|" + new Date().toLocaleString("en-GB") + "| ", ...args);
            })();
        }

        // creating missing folders
        if (!fs.existsSync(process.env.static_defaults)) {
            fs.mkdirSync(process.env.static_defaults);
            console.log("Created missing folder:", process.env.static_defaults + "\n", "Because this folder is now empty, it may cause errors with future requests.");
        }
        if (!fs.existsSync(process.env.static_uploads)) {
            fs.mkdirSync(process.env.static_uploads);
            console.log("Created missing folder:", process.env.static_uploads);
        }
        if (!fs.existsSync(process.env.static_public)) {
            fs.mkdirSync(process.env.static_public);
            console.log("Created missing folder:", process.env.static_public + "\n", "Because this folder is now empty, it may cause errors with building.");
        }

        // db, auth, AI inits
        await require("./db/db").init();
        await require("./routes/auth").init();
        await require("./routes/AI").init();

        // middleware
        app.use(cors());
        app.use(express.json());
        app.use(
            session({
                secret: process.env.SESSION_SECRET || "",
                cookie: { maxAge: parseInt(process.env.SESSION_MAX_AGE || 0) },
                resave: false,
                saveUninitialized: false,
                store: await require("./db/db").dbConfig.store,
            })
        );
        app.use(checkAuthToken);

        // routes
        app.use("/api/auth", require("./routes/auth").router);
        app.use("/api/user", require("./routes/user").router);
        app.use("/api/content", require("./routes/content").router);
        app.use("/api/img", require("./routes/img").router);
        app.use("/api/activities", require("./routes/activities").router);
        app.use("/api/gconfig", require("./routes/gconfig").router);
        app.use("/api/ai", require("./routes/AI").router);
        // static
        app.use(express.static(process.env.static_public));
        app.use("/defaults", express.static(process.env.static_defaults, { fallthrough: false }));
        app.use("/uploads", express.static(process.env.static_uploads, { fallthrough: false }));
        // redirect request on subpath to the app
        app.get("/themes", (req, res) => {
            res.sendFile(path.join(process.env.static_public, "themes.css"));
        });
        app.get("*", (req, res) => {
            res.sendFile(path.join(process.env.static_public, "index.html"));
        });

        // backups init & static build
        await require("./backup/backup").schedule();
        await require("./builder/builder").build();

        /**
         * Middleware for top-level error handling
         * @param {Object} err
         * @param {Object} req
         * @param {Object} res
         * @param {Function} next
         */
        app.use((err, req, res, next) => {
            // if (global.debug) console.error("ERROR HANDLING...\n", err.stack || err);
            switch (err.status) {
                case 404:
                    if (/(\/|\\)(defaults|uploads)(\/|\\)/.test(err.path)) {
                        const img = err.path.split(/(\/|\\).+(\/|\\)/).pop();
                        if (/^logo.*$/.test(img)) return res.sendFile(path.join(process.env.static_defaults, "defaultLogo.webp"));
                        else if (/^favicon.*$/.test(img)) return res.sendFile(path.join(process.env.static_defaults, "defaultFavicon.webp"));
                        else return res.sendFile(path.join(process.env.static_defaults, "defaultCover.png"));
                    }
                    break;
                default:
                    console.log("Internal server error: ", err);
                    res.status(500).send({ err: "Internal Server Error: " + err });
            }
        });
    } catch (err) {
        console.error("INIT ERROR...");
        console.log(err);
    }
    console.groupEnd();
}

init().then(() => {
    const PORT = process.env.PORT;
    app.listen(PORT, (err) => {
        if (err) return console.error("ERROR STARTING SERVER...\n", err);
        console.log(`Server listening on port ${PORT}\n`);
        // console.log = () => {
        //     throw Error();
        // };
    });
});
