import { createStore, createLogger } from "vuex";
import { getLocaleIndex, i18n } from "./translation.js";
const t = i18n.global.t;

export default createStore({
    state: {
        user: { role: "user", globalNotes: new Map() },
        mainMenu: {},
        activeTreeMenu: {},
        protectedSlugs: ["/home", "/activities", "/globalconfig", "/search"],
        articles: new Map(),
        textbooks: new Map(),
        activities: new Map(),
    },
    mutations: {
        // user
        userLogin(state, data) {
            data.globalNotes = new Map(Object.entries(data.globalNotes));
            state.user = data;

            let logout = setTimeout(() => this.commit("userLogout"), import.meta.env.VITE_APP_SESSION_MAX_AGE);
            if (["admin"].includes(data.role)) {
                setTimeout(() => {
                    window.openQuestionDialog({
                        title: t("components.dialogs.questions.logoutAuto.title"),
                        message: t("components.dialogs.questions.logoutAuto.message"),
                        actionButtons: [
                            {
                                text: t("components.dialogs.questions.logoutAuto.actionButtons.0"),
                                click: () => {
                                    document.getElementById("login").click();
                                    clearTimeout(logout);
                                },
                            },
                        ],
                    });
                }, import.meta.env.VITE_APP_SESSION_MAX_AGE - 10 * 60 * 1000); // -10min
            }
        },
        userLogout(state) {
            state.user = { role: "user", globalNotes: new Map() };
            window.isDirty = false;
            window.openToast({ title: t("components.dialogs.toasts.logoutWarn.title"), type: "warn", timeout: 2500 });
        },
        // menus
        updateMainMenu(state, data) {
            state.mainMenu = data;
        },
        updateActiveTreeMenu(state, data) {
            state.activeTreeMenu = data;
        },
        // articles
        updateArticles(state, data) {
            data.forEach(({ slug, _id, title }) => {
                state.articles.set(slug, { articleId: _id, title });
            });
        },
        setArticle(state, { slug, articleId, title } = data) {
            for (let key of state.articles.keys()) {
                if (state.articles.get(key).articleId === articleId) state.articles.delete(key);
            }
            state.articles.set(slug, { articleId, title });
        },
        deleteArticle(state, data) {
            state.articles.delete(data);
        },
        // textbooks
        updateTextbooks(state, data) {
            data.forEach(({ slug, title, articleId }) => {
                state.textbooks.set(slug, { title, articleId });
            });
        },
        setTextbook(state, { slug, title, articleId } = data) {
            for (let key of state.textbooks.keys()) {
                if (state.textbooks.get(key).articleId === articleId) state.textbooks.delete(key);
            }
            state.textbooks.set(slug, { title, articleId });
        },
        deleteTextbook(state, data) {
            state.textbooks.delete(data);
        },
        // activities
        updateActivities(state, data) {
            data.forEach(({ slug, _id, type, title, description, createdAt }) => {
                state.activities.set(slug, { activityId: _id, type, title, description, createdAt });
            });
        },
        setActivity(state, { slug, activityId, type, title, description, createdAt } = data) {
            for (let key of state.activities.keys()) {
                if (state.activities.get(key).activityId === activityId) state.activities.delete(key);
            }
            state.activities.set(slug, { activityId, type, title, description, createdAt });
        },
        deleteActivity(state, data) {
            state.activities.delete(data);
        },
    },
    getters: {
        /**
         * Returns if current user has required role (and therefore permission)
         * @param {String|Array} value string if only one role should be checked, array if more are possible
         * @returns true/false
         */
        hasPermission: (state) => (value) => {
            if (typeof value === "string") value = [value];
            return value.includes(state.user.role);
        },
        /**
         * @param {String} [only] pointer/screenSize
         * @returns if the device is mobile type
         */
        isMobileDevice:
            (state) =>
            (only = "pointer") => {
                const byPointer = window.matchMedia("(any-pointer: coarse)").matches;
                const byScreenSize = window.matchMedia("only screen and (max-width: 660px)").matches;

                if (only === "pointer") return byPointer;
                else if (only === "screenSize") return byScreenSize;
                else return byPointer || byScreenSize;
            },
        /**
         * Generates random unique ID
         * @param {Object} payload
         * @param {Number} [payload.base] tells to which base convert decimals
         * @param {Number} [payload.length] how long the ID should be
         * @param {string} [payload.checkString] CSS selector to check against
         * @returns ID
         */
        uniqueID:
            (state) =>
            ({ base = 16, length = 8, checkString = "[id='{{UID}}']" } = {}) => {
                let id = "";
                while (!id || document.querySelectorAll(checkString).length > 0) {
                    const arr = new Uint8Array(length / 2);
                    window.crypto.getRandomValues(arr);
                    id = Array.from(arr, (dec) => dec.toString(base).padStart(2, "0")).join("");
                    if (!/^[a-zA-Z]/.test(id)) id = "I" + id.substring(1); // valid HTML id must start with a letter
                    checkString = checkString.replaceAll(/\{\{UID\}\}/g, id);
                }
                return id;
            },
        fileSizeToString: (state) => (size) => {
            const i = size === 0 ? 0 : Math.floor(Math.log(size) / Math.log(1024));
            return (size / Math.pow(1024, i)).toFixed(2) * 1 + " " + ["B", "kB", "MB", "GB", "TB"][i];
        },
    },
    actions: {
        /**
         * Function for API requests
         * @param {Object} payload
         * @param {string} [payload.method]
         * @param {string} [payload.url]
         * @param {string} [payload.reqType]
         * @param {Object} [payload.additionalHeaders]
         * @param {Object} [payload.body]
         * @param {string} [payload.resType]
         * @returns {Promise<Object>} promise
         */
        async fetchAPI(context, { method = "GET", url = "/", reqType = "application/json", additionalHeaders = {}, body = {}, resType = "JSON" }) {
            return fetch(`/api${url}`, {
                method,
                headers: {
                    ...(reqType != "unset" && { "Content-Type": reqType }),
                    credentials: "include",
                    locale: getLocaleIndex(i18n.global.locale.value),
                    ...additionalHeaders,
                },
                ...(!/^(GET|HEAD)$/.test(method) && {
                    body: reqType === "application/json" ? JSON.stringify(body) : body,
                }),
            })
                .then(async (res) => {
                    let data;
                    switch (res.ok ? resType : "JSON") {
                        case "BLOB":
                            data = await res.blob();
                            break;
                        case "ARRAYBUFFER":
                            data = await res.arrayBuffer();
                            break;
                        case "FORMDATA":
                            data = await res.formData();
                            break;
                        case "TEXT":
                            data = await res.text();
                            break;
                        case "JSON":
                        default:
                            data = await res.json();
                    }
                    return { status: res.status, ok: res.ok, body: data };
                })
                .catch((err) => {
                    console.error("FETCH error...\n" + err);
                    window.showError(500, err);
                    return { ok: false, status: 500, body: { err } };
                });
        },
        /**
         * Function for removing article
         * @param {Object} payload
         * @param {string} payload.articleId
         * @param {string} payload.slug
         * @returns {boolean} if everything went ok and the job on the server was done
         */
        async deleteArticle(context, { articleId, slug, menuName }) {
            const res = await this.dispatch("fetchAPI", {
                method: "DELETE",
                url: "/content/article/" + articleId,
            });
            switch (res.status) {
                case 409:
                    if (res.body.err.menus.some((m) => m !== menuName)) {
                        await window.openQuestionDialog({
                            title: t("components.dialogs.questions.menu404Delete.title"),
                            message: "Menu: " + res.body.err.menus.join(", ") + " " + t("components.dialogs.questions.menu404Delete.message"),
                            actionButtons: [
                                {
                                    text: t("components.dialogs.questions.menu404Delete.actionButtons.0"),
                                    click: async () => {
                                        await this.dispatch("fetchAPI", {
                                            method: "POST",
                                            url: "/content/menu/clean",
                                        }).then((res) => {
                                            if (!res.ok) return window.showError(res.status, res.body.err);
                                        });
                                    },
                                },
                            ],
                            cancelButton: {
                                text: t("components.dialogs.questions.menu404Delete.cancelButton"),
                                click: () => {},
                            },
                        });
                    }
                    this.commit("deleteArticle", slug);
                    window.reloadMenus();
                    break;
                case 406:
                    await window.openToast({
                        title: t("components.dialogs.toasts.warn.title"),
                        message: res.body.err,
                        type: "warn",
                    });
                    return false;
                default:
                    if (!res.ok) {
                        await window.openToast({
                            title: t("components.dialogs.toasts.deleteError.title"),
                            message: res.body.err,
                            type: "err",
                        });
                        return false;
                    }
            }

            this.commit("deleteArticle", slug);
            window.reloadMenus();

            //TODO
            //if current article je ten smazaný > router.replace("/")
            //potom v articleEditor v remove() smazat přesměrování
            return true;
        },
        /**
         * Function for removing textbook
         * @param {Object} payload
         * @param {string} payload.slug
         * @returns {boolean} if everything went ok
         */
        async deleteTextBook(context, { articleId, slug }) {
            const res = await this.dispatch("fetchAPI", {
                method: "DELETE",
                url: "/content/menu" + slug,
            });
            if (!res.ok) {
                window.openToast({
                    title: t("components.dialogs.toasts.deleteError.title"),
                    message: res.body.err,
                    type: "err",
                });
            }
            if (!(await this.dispatch("deleteArticle", { articleId, slug }))) return false;
            this.commit("deleteTextbook", slug);
            return true;
        },
        async translateArticle(ctx, { source, lang }) {
            const res = await ctx.dispatch("fetchAPI", { method: "POST", url: "/ai/translate/", body: { source, lang } });
            if (!res.ok) {
                window.openToast({
                    title: t("components.dialogs.toasts.translateArticleError.title"),
                    message: res.body.err,
                    type: "err",
                });
                return;
            }
            return res.body.translation;
        },
        async generateSummary(ctx, text) {
            const res = await ctx.dispatch("fetchAPI", { method: "POST", url: "/ai/summary/", body: { text } });
            if (!res.ok) {
                window.openToast({
                    title: t("components.dialogs.toasts.createSummaryError.title"),
                    message: res.body.err,
                    type: "err",
                });
                return;
            }
            return res.body.summary;
        },
    },
    plugins: [
        createLogger({
            collapsed: true,
            logActions: false,
            logMutations: process.env.NODE_ENV === "development",
        }),
    ],
});
